package cn.iocoder.yudao.module.shipment.listener;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.apollo.core.event.export.ShipmentReceivableExcelExportPushEvent2;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseDO;
import cn.iocoder.yudao.module.depository.dal.dataobject.warehouse.WarehouseLineDO;
import cn.iocoder.yudao.module.depository.dal.mysql.warehouse.WarehouseLineMapper;
import cn.iocoder.yudao.module.depository.dto.ContactInfoDTO;
import cn.iocoder.yudao.module.depository.service.warehouse.WarehouseService;
import cn.iocoder.yudao.module.ecw.api.currency.CurrencyApi;
import cn.iocoder.yudao.module.ecw.api.currency.dto.ExchangeRateRespDTO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.currency.CurrencyDO;
import cn.iocoder.yudao.module.ecw.dal.dataobject.region.RegionDO;
import cn.iocoder.yudao.module.ecw.service.currency.CurrencyService;
import cn.iocoder.yudao.module.ecw.service.region.RegionService;
import cn.iocoder.yudao.module.infra.dal.dataobject.file.FileDO;
import cn.iocoder.yudao.module.infra.service.file.FileService;
import cn.iocoder.yudao.module.order.dto.OrderBackInfoDto;
import cn.iocoder.yudao.module.order.service.order.*;
import cn.iocoder.yudao.module.order.vo.orderItem.OrderItemBackVO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxMergePkgDO;
import cn.iocoder.yudao.module.shipment.dal.dataobject.BoxPkgOrderDO;
import cn.iocoder.yudao.module.shipment.dto.ImageData;
import cn.iocoder.yudao.module.shipment.service.box.BoxService;
import cn.iocoder.yudao.module.shipment.service.boxMergePkg.BoxMergePkgService;
import cn.iocoder.yudao.module.shipment.service.boxPkgOrder.BoxPkgOrderService;
import cn.iocoder.yudao.module.shipment.service.boxPreloadGoods.BoxPreloadGoodsService;
import cn.iocoder.yudao.module.shipment.vo.boxPreloadGoods.BoxPreloadGoodsBackVO;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.framework.ue.UeProperties;
import cn.iocoder.yudao.module.wealth.service.receivable.ReceivableService;
import cn.iocoder.yudao.module.wealth.vo.receivable.ReceivableOrderPayedFeeTypeVO;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.util.FileUtils;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.metadata.fill.FillConfig;
import com.alibaba.fastjson.JSONObject;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.context.event.EventListener;
import org.springframework.security.config.annotation.web.configurers.UrlAuthorizationConfigurer;
import org.springframework.security.task.DelegatingSecurityContextAsyncTaskExecutor;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.io.*;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.excel.constant.ExportConstant.DATA_FORMAT;
import static cn.iocoder.yudao.module.shipment.enums.ErrorCodeConstants.HAVE_NOT_PRELOAD_GOODS;

import java.net.URL;

/**
 * 自编号订单excel导出监听
 * 海运应收汇总表单
 *
 * @author
 */
@Component("ShipmentReceivableExcelExportListener2")
@AllArgsConstructor
@Slf4j
public class ShipmentReceivableExcelExportListener2 {

    private final OrderQueryService orderQueryService;
    private UeProperties ueProperties;
    private FileService fileService;

    @Resource
    private BoxService boxService;

    @Resource
    private BoxPreloadGoodsService boxPreloadGoodsService;

    @Resource
    private WarehouseLineMapper warehouseLineMapper;

    @Resource
    private WarehouseService warehouseService;

    @Resource
    private RegionService regionService;

    @Resource
    private CurrencyService currencyService;

    @Resource
    private ReceivableService receivableService;

    @Resource
    private CurrencyApi currencyApi;

    @Resource
    private BoxPkgOrderService boxPkgOrderService;

    @Resource
    private BoxMergePkgService boxMergePkgService;

    @Resource
    private AdminUserApi adminUserApi;

    /*
     */
    private String getTemplateName(WarehouseLineDO warehouseLineDO) {
        String sTemName = "/nrlyReceivable2.xlsx";
        if (warehouseLineDO.getTzName() == null ||
                warehouseLineDO.getTzName().length() == 0) {
            sTemName = "/nrlyReceivable2.xlsx";
        } else {
            if (warehouseLineDO.getTzName().equals("广州市捷道国际货运代理有限公司")) {
                sTemName = "/nrlyReceivable2.xlsx";
            }
            else if (warehouseLineDO.getTzName().equals("广州市捷佳国际货运代理有限公司")) {
                sTemName = "/nrlyReceivable3.xlsx";
            }
            else if (warehouseLineDO.getTzName().equals("义乌市捷道国际货运代理有限公司")) {
                sTemName = "/nrlyReceivable4.xlsx";
            } else {
                sTemName = "/nrlyReceivable2.xlsx";
            }
        }
        return sTemName;
    }

    /**
     * 订单导出监听
     *
     * @param event 编号记录
     */
    @EventListener(ShipmentReceivableExcelExportPushEvent2.class)
    public void shipmentReceivableExcelExportPushEvent(
            ShipmentReceivableExcelExportPushEvent2 event) {

        if (StringUtils.isNotBlank(event.getRequestParams())) {
            try {
                ContactInfoDTO recipientInfo = new ContactInfoDTO();
                ContactInfoDTO senderInfo = new ContactInfoDTO();
                DateTimeFormatter formatter = DateTimeFormatter.ofPattern(DATA_FORMAT);
                String nowTime = formatter.format(LocalDateTime.now());
                String dir = ueProperties.getTempDir().concat("/receivable/excel/");


                //先不合并单元格，之后合并，把合并的传到服务器
                String fileName = event.getUserId().toString().
                        concat(StrUtil.DASHED).concat(event.getUserType().toString()).
                        concat(StrUtil.DASHED).concat(nowTime).
                        concat("receivable_notMerge.xlsx");

                String fileNameMerge = event.getUserId().toString().
                        concat(StrUtil.DASHED).concat(event.getUserType().toString()).
                        concat(StrUtil.DASHED).concat(nowTime).
                        concat("receivable_Merge.xlsx");

                List<CurrencyDO> currencyList = currencyService.getCurrencyList();
                Map<Integer, String> currencyMap =
                        currencyList.stream().
                                collect(Collectors.toMap(CurrencyDO::getId,
                                        CurrencyDO::getTitleEn, (key1, key2) -> key2));
                Map<Integer, String> currencyFuhaoMap =
                        currencyList.stream().
                                collect(Collectors.toMap(CurrencyDO::getId,
                                        CurrencyDO::getFuhao, (key1, key2) -> key2));
                InputStream inputStream = null;
                JSONObject jsonObject =
                        JSONObject.parseObject(event.getRequestParams());
                Long shipmentId = jsonObject.getLong("shipmentId");


                BoxDO box = boxService.getBox(shipmentId);
                WarehouseDO startWarehouse = warehouseService.getWarehouse(box.getStartWarehouseId());
                WarehouseDO destWarehouse = warehouseService.getWarehouse(box.getDestWarehouseId());
                RegionDO startRegion = regionService.getRegion(startWarehouse.getGuojia());
                RegionDO destRegion = regionService.getRegion(destWarehouse.getGuojia());
                WarehouseLineDO warehouseLineDO = warehouseLineMapper.findOne(
                        box.getStartWarehouseId(), box.getDestWarehouseId(),
                        box.getTransportType());

                // 添加动态模板逻辑
                String sTemName = getTemplateName(warehouseLineDO);
                inputStream = getClass().getClassLoader().
                        getResourceAsStream(ueProperties.getTemplatesUrl()
                                + sTemName);

                if (StringUtils.isNotBlank(warehouseLineDO.getRecipientInfo())) {
                    recipientInfo = JSONObject.parseObject(warehouseLineDO.getRecipientInfo(), ContactInfoDTO.class);
                }
                if (StringUtils.isNotBlank(warehouseLineDO.getSenderInfo())) {
                    senderInfo = JSONObject.parseObject(warehouseLineDO.getSenderInfo(), ContactInfoDTO.class);
                }
                Long currencyUnit = warehouseLineDO.getCurrencyUnit();
                Map headMap = new HashMap();
                Integer yfCurrency = Integer.valueOf(destRegion.getImportCurrency1());//运费币种
                Integer qgCurrency = Integer.valueOf(destRegion.getImportCurrency2());//清关费币种
                Integer otherMainCurrency = Integer.valueOf(destRegion.getImportCurrency3());//额外费用主币种
                Integer otherAssisCurrency = Integer.valueOf(destRegion.getImportCurrency4());//额外费用副币种
                headMap.put("tempTitleEn", StringUtils.isBlank(warehouseLineDO.getTempTitleEn()) ? "GUANGZHOU E&C LOGISTICS LIMITED" : warehouseLineDO.getTempTitleEn());
                headMap.put("tempTitleZh", StringUtils.isBlank(warehouseLineDO.getTempTitleZh()) ? "广州市捷道国际货运代理有限公司" : warehouseLineDO.getTempTitleZh());


                if (StringUtils.isBlank(warehouseLineDO.getLkLeft())) {
                    headMap.put("lkLeft", "落款参数未配置");
                } else {
                    headMap.put("lkLeft", warehouseLineDO.getLkLeft());
                }

                if (StringUtils.isBlank(warehouseLineDO.getLkRight())) {
                    headMap.put("lkRight", "落款参数未配置");
                } else {
                    headMap.put("lkRight", warehouseLineDO.getLkRight());
                }

                // 添加图片自动填充功能
                ImageData imageData = new ImageData();
                imageData.setUrl(new URL(
                        "https://jiedao-pre-production.oss-cn-shenzhen.aliyuncs.com/2024/06/11/6668347de4b001393be98e24.jpg"));
                headMap.put("img1", imageData.getUrl());
                headMap.put("img2", imageData.getUrl());
                //end  添加图片自动填充功能

                if (box.getZgDate() != null) {
                    //装柜日期  日期格式化
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                    String currentDate = sdf.format(box.getZgDate());
                    headMap.put("zkDate", currentDate);
                }
                headMap.put("selfNo", box.getSelfNo());
                headMap.put("fromZh", senderInfo.getName());
                headMap.put("fromEn", senderInfo.getNameEn());
                headMap.put("fromAddressZh", senderInfo.getCompanyAddress());
                headMap.put("fromAddressEn", senderInfo.getCompanyAddressEn());
                headMap.put("toZh", recipientInfo.getName());
                headMap.put("toEn", recipientInfo.getNameEn());
                headMap.put("toAddressZh", recipientInfo.getCompanyAddress());
                headMap.put("toAddressEn", recipientInfo.getCompanyAddressEn());
                headMap.put("currency1", currencyMap.get(yfCurrency));
                headMap.put("currency2", currencyMap.get(qgCurrency));
                headMap.put("currency3", currencyMap.get(otherMainCurrency));
                headMap.put("currency4", currencyMap.get(otherAssisCurrency));

                headMap.put("arrivedDate", DateUtil.format(new Date(), "yyyy-MM-dd"));

                headMap.put("fromCountryEn", startRegion.getTitleEn());
                headMap.put("fromCountryZh", startRegion.getTitleZh());
                headMap.put("toCountryEn", destRegion.getTitleEn());
                headMap.put("toCountryZh", destRegion.getTitleZh());
                List<BoxPreloadGoodsBackVO> goodsItemList = boxPreloadGoodsService.getShipGoodItemList(shipmentId);
                if (CollectionUtil.isEmpty(goodsItemList)) {
                    event.setResult(HAVE_NOT_PRELOAD_GOODS.getMsg());
                    return;
                }
                //以secId排序
                goodsItemList.sort(Comparator.comparing(BoxPreloadGoodsBackVO::getPkgId).reversed().thenComparing(BoxPreloadGoodsBackVO::getTidanNum));
//
                List<Map<String, Object>> list = new ArrayList<>();
                //以订单ID分组
                Map<Long, List<BoxPreloadGoodsBackVO>> orderMap = goodsItemList.stream()
                        .collect(Collectors.groupingBy(BoxPreloadGoodsBackVO::getOrderId, LinkedHashMap::new, Collectors.toList()));
                //所有订单的每列合计
                BigDecimal sum1 = BigDecimal.ZERO;
                BigDecimal sum2 = BigDecimal.ZERO;
                BigDecimal sum3 = BigDecimal.ZERO;
                BigDecimal sum4 = BigDecimal.ZERO;
                BigDecimal sum5 = BigDecimal.ZERO;
                BigDecimal sum6 = BigDecimal.ZERO;
                BigDecimal sum7 = BigDecimal.ZERO;
                BigDecimal sum8 = BigDecimal.ZERO;
                BigDecimal sum9 = BigDecimal.ZERO;
                BigDecimal sum10 = BigDecimal.ZERO;
                BigDecimal sum11 = BigDecimal.ZERO;
                BigDecimal sum12 = BigDecimal.ZERO;
                BigDecimal sum13 = BigDecimal.ZERO;
                Integer sum14 = 0;
                BigDecimal sum15 = BigDecimal.ZERO;
                BigDecimal sum16 = BigDecimal.ZERO;
                List<BoxMergePkgDO> boxMergePkgDOS =
                        boxMergePkgService.selectList(
                                new LambdaQueryWrapperX<BoxMergePkgDO>().
                                        eq(BoxMergePkgDO::getShipmentId,
                                                shipmentId).orderByDesc(BoxMergePkgDO::getId));

                for (Map.Entry<Long, List<BoxPreloadGoodsBackVO>> entry : orderMap.entrySet()) {
                    Long orderId = entry.getKey();
                    OrderBackInfoDto orderInfo = orderQueryService.info(orderId);
                    Map<String, Object> map = new HashMap<>();
                    map.put("consigneeName", orderInfo.getConsigneeVO().getName());
                    map.put("mark", orderInfo.getMarks());
                    map.put("orderNo", orderInfo.getOrderNo());
                    map.put("tidanNo", orderInfo.getTidanNo());
                    map.put("consigneePhone", "+" + orderInfo.getConsigneeVO().getCountryCode() + orderInfo.getConsigneeVO().getPhone());
                    //todo 计算应收明细的核销比例,待测试，未减优惠金额
                    List<ReceivableOrderPayedFeeTypeVO> payedFeeTypeVOList = receivableService.getPayedMoneyByFeeType(orderId);
                    //应收运费
                    List<ReceivableOrderPayedFeeTypeVO> collect = payedFeeTypeVOList.stream().filter(item -> item.getFeeType() == 1).collect(Collectors.toList());
                    BigDecimal changeAmountSum = collect.stream().filter(item -> item.getCurrencyId().longValue() != Long.valueOf(yfCurrency).longValue()).map(item -> {
                        BigDecimal toBigDecimal = changeAmountToDestCurrency(item.getCurrencyId(), Long.valueOf(yfCurrency), item.getTotalAmount());
                        return toBigDecimal;
                    }).reduce(BigDecimal.ZERO, BigDecimal::add);//运费不同时转换币种后求和
                    BigDecimal unChangeAmountSum = collect.stream().filter(item -> item.getCurrencyId().longValue() == Long.valueOf(yfCurrency).longValue()).map(ReceivableOrderPayedFeeTypeVO::getTotalAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    BigDecimal totalFreight = unChangeAmountSum.add(changeAmountSum);
                    totalFreight = totalFreight.setScale(2, RoundingMode.DOWN);
                    map.put("totalFreight", currencyFuhaoMap.get(yfCurrency) + totalFreight);
                    sum1 = sum1.add(totalFreight);
                    //应收清关费
                    List<ReceivableOrderPayedFeeTypeVO> clearingList = payedFeeTypeVOList.stream().filter(item -> item.getFeeType() == 2).collect(Collectors.toList());
                    BigDecimal changeClearingAmountSum = clearingList.stream().filter(item -> item.getCurrencyId().longValue() != Long.valueOf(qgCurrency).longValue()).map(item -> {
                        BigDecimal toBigDecimal = changeAmountToDestCurrency(item.getCurrencyId(), Long.valueOf(qgCurrency), item.getTotalAmount());
                        return toBigDecimal;
                    }).reduce(BigDecimal.ZERO, BigDecimal::add);//清关费不同时转换币种后求和
                    BigDecimal unChangeClearingAmountSum = clearingList.stream().filter(item -> item.getCurrencyId().longValue() == Long.valueOf(qgCurrency).longValue()).map(ReceivableOrderPayedFeeTypeVO::getTotalAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    BigDecimal totalClearing = unChangeClearingAmountSum.add(changeClearingAmountSum);
                    totalClearing = totalClearing.setScale(2, RoundingMode.DOWN);
                    map.put("totalClearing", currencyFuhaoMap.get(qgCurrency) + totalClearing);
                    sum2 = sum2.add(totalClearing);
                    //保价费
                    List<ReceivableOrderPayedFeeTypeVO> valuationList = payedFeeTypeVOList.stream().filter(item -> item.getFeeType() == 6).collect(Collectors.toList());
                    BigDecimal changeValuationAmountSum = valuationList.stream().filter(item -> item.getCurrencyId().longValue() != currencyUnit.longValue()).map(item -> {
                        BigDecimal toBigDecimal = changeAmountToDestCurrency(item.getCurrencyId(), currencyUnit, item.getTotalAmount());
                        return toBigDecimal;
                    }).reduce(BigDecimal.ZERO, BigDecimal::add);//保价费不同时转换币种后求和
                    BigDecimal unChangeValuationAmountSum = valuationList.stream().filter(item -> item.getCurrencyId().longValue() == currencyUnit.longValue()).map(ReceivableOrderPayedFeeTypeVO::getTotalAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    BigDecimal totalValuation = unChangeValuationAmountSum.add(changeValuationAmountSum);
                    map.put("totalValuation", (currencyFuhaoMap.get(currencyUnit) == null ? "" : currencyFuhaoMap.get(currencyUnit)) + totalValuation);
                    sum3 = sum3.add(totalValuation);
                    //额外费用
                    List<ReceivableOrderPayedFeeTypeVO> otherFeeList = payedFeeTypeVOList.stream().filter(item -> item.getFeeType() != 1 && item.getFeeType() != 2 && item.getFeeType() != 6).collect(Collectors.toList());
                    BigDecimal changeOtherMainAmountSum = otherFeeList.stream().filter(item -> item.getCurrencyId().longValue() != Long.valueOf(otherAssisCurrency).longValue() && item.getCurrencyId().longValue() != Long.valueOf(otherMainCurrency).longValue()).map(item -> {
                        BigDecimal toBigDecimal = changeAmountToDestCurrency(item.getCurrencyId(), Long.valueOf(otherMainCurrency), item.getTotalAmount());
                        return toBigDecimal;
                    }).reduce(BigDecimal.ZERO, BigDecimal::add);//额外费不同时转换币种后求和
                    BigDecimal unChangeOtherMainAmountSum = otherFeeList.stream().filter(item -> item.getCurrencyId().longValue() == Long.valueOf(otherMainCurrency).longValue()).map(ReceivableOrderPayedFeeTypeVO::getTotalAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    BigDecimal unChangeOtherAssisAmountSum = otherFeeList.stream().filter(item -> item.getCurrencyId().longValue() == Long.valueOf(otherAssisCurrency).longValue()).map(ReceivableOrderPayedFeeTypeVO::getTotalAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    BigDecimal totalOtherMain = unChangeOtherMainAmountSum.add(changeOtherMainAmountSum);
                    totalOtherMain = totalOtherMain.setScale(2, RoundingMode.DOWN);
                    map.put("totalOtherMain", currencyFuhaoMap.get(otherMainCurrency) + totalOtherMain);
                    sum4 = sum4.add(totalOtherMain);
                    unChangeOtherAssisAmountSum = unChangeOtherAssisAmountSum.setScale(2, RoundingMode.DOWN);
                    map.put("totalOtherAssis", currencyFuhaoMap.get(otherAssisCurrency) + unChangeOtherAssisAmountSum);
                    sum5 = sum5.add(unChangeOtherAssisAmountSum);
                    BigDecimal totalOrderAmountLeft = BigDecimal.ZERO;
                    BigDecimal totalOrderAmountRight = BigDecimal.ZERO;
                    totalOrderAmountLeft = totalOrderAmountLeft.add(totalFreight);
                    totalOrderAmountRight = totalOrderAmountRight.add(totalClearing);
                    //总计加保价费
                    if (currencyUnit.longValue() == Long.valueOf(yfCurrency).longValue()) {
                        totalOrderAmountLeft = totalOrderAmountLeft.add(totalValuation);
                    } else if (currencyUnit.longValue() == Long.valueOf(qgCurrency).longValue()) {
                        totalOrderAmountRight = totalOrderAmountRight.add(totalValuation);
                    } else {
                        BigDecimal changedAmount = changeAmountToDestCurrency(currencyUnit, Long.valueOf(yfCurrency), totalValuation);
                        totalOrderAmountLeft = totalOrderAmountLeft.add(changedAmount);
                    }
                    if (Long.valueOf(otherMainCurrency).longValue() == Long.valueOf(otherAssisCurrency).longValue()) {
                        //总计加额外费用主币种
                        if (Long.valueOf(otherMainCurrency).longValue() == Long.valueOf(yfCurrency).longValue()) {
                            totalOrderAmountLeft = totalOrderAmountLeft.add(totalOtherMain);
                        } else if (Long.valueOf(otherMainCurrency).longValue() == Long.valueOf(qgCurrency).longValue()) {
                            totalOrderAmountRight = totalOrderAmountRight.add(totalOtherMain);
                        } else {
                            BigDecimal changedAmount = changeAmountToDestCurrency(Long.valueOf(otherMainCurrency), Long.valueOf(yfCurrency), totalOtherMain);
                            totalOrderAmountLeft = totalOrderAmountLeft.add(changedAmount);
                        }
                    } else {
                        //总计加额外费用主币种
                        if (Long.valueOf(otherMainCurrency).longValue() == Long.valueOf(yfCurrency).longValue()) {
                            totalOrderAmountLeft = totalOrderAmountLeft.add(totalOtherMain);
                        } else if (Long.valueOf(otherMainCurrency).longValue() == Long.valueOf(qgCurrency).longValue()) {
                            totalOrderAmountRight = totalOrderAmountRight.add(totalOtherMain);
                        } else {
                            BigDecimal changedAmount = changeAmountToDestCurrency(Long.valueOf(otherMainCurrency), Long.valueOf(yfCurrency), totalOtherMain);
                            totalOrderAmountLeft = totalOrderAmountLeft.add(changedAmount);
                        }
                        //总计加额外费用副币种
                        if (Long.valueOf(otherAssisCurrency).longValue() == Long.valueOf(yfCurrency).longValue()) {
                            totalOrderAmountLeft = totalOrderAmountLeft.add(unChangeOtherAssisAmountSum);
                        } else if (Long.valueOf(otherAssisCurrency).longValue() == Long.valueOf(qgCurrency).longValue()) {
                            totalOrderAmountRight = totalOrderAmountRight.add(unChangeOtherAssisAmountSum);
                        } else {
                            BigDecimal changedAmount = changeAmountToDestCurrency(Long.valueOf(otherAssisCurrency), Long.valueOf(yfCurrency), unChangeOtherAssisAmountSum);
                            totalOrderAmountLeft = totalOrderAmountLeft.add(changedAmount);
                        }
                    }
                    totalOrderAmountLeft = totalOrderAmountLeft.setScale(2, RoundingMode.DOWN);
                    map.put("totalOrderAmountLeft", currencyFuhaoMap.get(yfCurrency) + totalOrderAmountLeft);
                    sum6 = sum6.add(totalOrderAmountLeft);
                    totalOrderAmountRight = totalOrderAmountRight.setScale(2, RoundingMode.DOWN);
                    map.put("totalOrderAmountRight", currencyFuhaoMap.get(qgCurrency) + totalOrderAmountRight);
                    sum7 = sum7.add(totalOrderAmountRight);
                    //中国预付运费
                    List<ReceivableOrderPayedFeeTypeVO> fromPaymentFreightList = payedFeeTypeVOList.stream().filter(item -> item.getCollectionType() != null && item.getCollectionType() == 1 && item.getFeeType() == 1).collect(Collectors.toList());
                    BigDecimal changeFromPaymentFreightSum = fromPaymentFreightList.stream().filter(item -> item.getCurrencyId().longValue() != Long.valueOf(yfCurrency).longValue()).map(item -> {
                        BigDecimal toBigDecimal = changeAmountToDestCurrency(item.getCurrencyId(), Long.valueOf(yfCurrency), item.getPayedAmount());
                        return toBigDecimal;
                    }).reduce(BigDecimal.ZERO, BigDecimal::add);//始发预付运费不同时转换币种后求和
                    BigDecimal unChangeFromPaymentFreightSum = fromPaymentFreightList.stream().filter(item -> item.getCurrencyId().longValue() == Long.valueOf(yfCurrency).longValue()).map(ReceivableOrderPayedFeeTypeVO::getPayedAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    BigDecimal totalFromPaymentFreight = unChangeFromPaymentFreightSum.add(changeFromPaymentFreightSum);
                    totalFromPaymentFreight = totalFromPaymentFreight.setScale(2, RoundingMode.DOWN);
                    map.put("totalFromPaymentFreight", currencyFuhaoMap.get(yfCurrency) + totalFromPaymentFreight);
                    sum8 = sum8.add(totalFromPaymentFreight);
                    //中国预付清关费
                    List<ReceivableOrderPayedFeeTypeVO> fromPaymentClearingList = payedFeeTypeVOList.stream().filter(item -> item.getCollectionType() != null && item.getCollectionType() == 1 && item.getFeeType() == 2).collect(Collectors.toList());
                    BigDecimal changeFromPaymentClearingSum = fromPaymentClearingList.stream().filter(item -> item.getCurrencyId().longValue() != Long.valueOf(yfCurrency).longValue()).map(item -> {
                        BigDecimal toBigDecimal = changeAmountToDestCurrency(item.getCurrencyId(), Long.valueOf(yfCurrency), item.getPayedAmount());
                        return toBigDecimal;
                    }).reduce(BigDecimal.ZERO, BigDecimal::add);//始发预付清关费不同时转换币种后求和
                    BigDecimal unChangeFromPaymentClearingSum = fromPaymentClearingList.stream().filter(item -> item.getCurrencyId().longValue() == Long.valueOf(yfCurrency).longValue()).map(ReceivableOrderPayedFeeTypeVO::getPayedAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    BigDecimal totalFromPaymentClearing = unChangeFromPaymentClearingSum.add(changeFromPaymentClearingSum);
                    totalFromPaymentClearing = totalFromPaymentClearing.setScale(2, RoundingMode.DOWN);
                    map.put("totalFromPaymentClearing", currencyFuhaoMap.get(qgCurrency) + totalFromPaymentClearing);
                    sum9 = sum9.add(totalFromPaymentClearing);
                    //目的国预付运费
                    List<ReceivableOrderPayedFeeTypeVO> toPaymentFreightList = payedFeeTypeVOList.stream().filter(item -> item.getCollectionType() != null && item.getCollectionType() == 2 && item.getFeeType() == 1).collect(Collectors.toList());
                    if (CollectionUtil.isEmpty(toPaymentFreightList)) {
                        toPaymentFreightList = payedFeeTypeVOList.stream().filter(item -> item.getCollectionType() != null && item.getCollectionType() == 3 && item.getFeeType() == 1).collect(Collectors.toList());
                    }
                    BigDecimal changeToPaymentFreightSum = toPaymentFreightList.stream().filter(item -> item.getCurrencyId().longValue() != Long.valueOf(yfCurrency).longValue()).map(item -> {
                        BigDecimal toBigDecimal = changeAmountToDestCurrency(item.getCurrencyId(), Long.valueOf(yfCurrency), item.getPayedAmount());
                        return toBigDecimal;
                    }).reduce(BigDecimal.ZERO, BigDecimal::add);//始发预付运费不同时转换币种后求和
                    BigDecimal unChangeToPaymentFreightSum = toPaymentFreightList.stream().filter(item -> item.getCurrencyId().longValue() == Long.valueOf(yfCurrency).longValue()).map(ReceivableOrderPayedFeeTypeVO::getPayedAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    BigDecimal totalToPaymentFreight = unChangeToPaymentFreightSum.add(changeToPaymentFreightSum);
                    totalToPaymentFreight = totalToPaymentFreight.setScale(2, RoundingMode.DOWN);
                    map.put("totalToPaymentFreight", currencyFuhaoMap.get(yfCurrency) + totalToPaymentFreight);
                    sum10 = sum10.add(totalToPaymentFreight);
                    //目的国预付清关费
                    List<ReceivableOrderPayedFeeTypeVO> toPaymentClearingList = payedFeeTypeVOList.stream().filter(item -> item.getCollectionType() != null && item.getCollectionType() == 2 && item.getFeeType() == 2).collect(Collectors.toList());
                    if (CollectionUtil.isEmpty(toPaymentFreightList)) {
                        toPaymentClearingList = payedFeeTypeVOList.stream().filter(item -> item.getCollectionType() != null && item.getCollectionType() == 3 && item.getFeeType() == 2).collect(Collectors.toList());
                    }
                    BigDecimal changeToPaymentClearingSum = toPaymentClearingList.stream().filter(item -> item.getCurrencyId().longValue() != Long.valueOf(qgCurrency).longValue()).map(item -> {
                        BigDecimal toBigDecimal = changeAmountToDestCurrency(item.getCurrencyId(), Long.valueOf(qgCurrency), item.getPayedAmount());
                        return toBigDecimal;
                    }).reduce(BigDecimal.ZERO, BigDecimal::add);//始发预付运费不同时转换币种后求和
                    BigDecimal unChangeToPaymentClearingSum = toPaymentClearingList.stream().filter(item -> item.getCurrencyId().longValue() == Long.valueOf(qgCurrency).longValue()).map(ReceivableOrderPayedFeeTypeVO::getPayedAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
                    BigDecimal totalToPaymentClearing = unChangeToPaymentClearingSum.add(changeToPaymentClearingSum);
                    totalToPaymentClearing = totalToPaymentClearing.setScale(2, RoundingMode.DOWN);
                    map.put("totalToPaymentClearing", currencyFuhaoMap.get(qgCurrency) + totalToPaymentClearing);
                    sum11 = sum11.add(totalToPaymentClearing);
                    //目的地应收
                    BigDecimal orderReceivableAmountLeft = totalOrderAmountLeft.subtract(totalToPaymentFreight).subtract(totalFromPaymentFreight);
                    BigDecimal orderReceivableAmountRight = totalOrderAmountRight.subtract(totalToPaymentClearing).subtract(totalFromPaymentClearing);
                    orderReceivableAmountLeft = orderReceivableAmountLeft.setScale(2, RoundingMode.DOWN);
                    map.put("orderReceivableAmountLeft", currencyFuhaoMap.get(yfCurrency) + orderReceivableAmountLeft);
                    sum12 = sum12.add(orderReceivableAmountLeft);
                    orderReceivableAmountRight = orderReceivableAmountRight.setScale(2, RoundingMode.DOWN);
                    map.put("orderReceivableAmountRight", currencyFuhaoMap.get(qgCurrency) + orderReceivableAmountRight);
                    sum13 = sum13.add(orderReceivableAmountRight);

                    Long existInMergePkg = boxPkgOrderService.selectCount(BoxPkgOrderDO::getOrderId, orderId);
                    if (existInMergePkg != null && existInMergePkg > 0L) {
//                        map.put("sumNum", boxMergePkgDOS.size());
                        map.put("sumNum", "1");
                    } else {
                        map.put("sumNum", orderInfo.getSumNum());
                        sum14 = sum14 + orderInfo.getSumNum();
                    }
                    BigDecimal volume = orderInfo.getOrderItemVOList().stream().map(OrderItemBackVO::getChargeVolume).reduce(BigDecimal.ZERO, BigDecimal::add);
                    map.put("sumVolume", volume);
//                    map.put("sumVolume", orderInfo.getSumVolume());
                    sum15 = sum15.add(volume);
                    map.put("sumWeight", orderInfo.getSumWeight());
                    sum16 = sum16.add(orderInfo.getSumWeight());
                    map.put("vWeight", orderInfo.getVWeight());
                    map.put("isCargoControl", orderInfo.getIsCargoControl() ? "YES" : "NO");
                    map.put("objectiveName", orderInfo.getLogisticsInfoDto().getDestTitleEn());
                    map.put("consignorPhone", orderInfo.getConsignorVO().getPhone());
                    map.put("consignorName", orderInfo.getConsignorVO().getName());

                    if (CollectionUtil.isNotEmpty(entry.getValue())) {
                        Long pkgId = entry.getValue().get(0).getPkgId();
                        if (pkgId != null && pkgId > 0L) {
                            BoxMergePkgDO boxMergePkgDO = boxMergePkgService.selectOne(BoxMergePkgDO::getId, pkgId);
                            map.put("remark", boxMergePkgDO.getPkgNum());
                        }
                    }
                    if (orderInfo.getSalesmanId() != null &&
                            orderInfo.getSalesmanId().longValue() > 0L) {
                        map.put("accountManager",
                                getUserName(String.valueOf(orderInfo.getSalesmanId())));
                    }
                    list.add(map);
                }
                headMap.put("sum1", currencyFuhaoMap.get(yfCurrency) + sum1);
                headMap.put("sum2", currencyFuhaoMap.get(qgCurrency) + sum2);
                headMap.put("sum3", (currencyFuhaoMap.get(currencyUnit) == null ? "" : currencyFuhaoMap.get(currencyUnit)) + sum3);
                headMap.put("sum4", currencyFuhaoMap.get(otherMainCurrency) + sum4);
                headMap.put("sum5", currencyFuhaoMap.get(otherAssisCurrency) + sum5);
                headMap.put("sum6", currencyFuhaoMap.get(yfCurrency) + sum6);
                headMap.put("sum7", currencyFuhaoMap.get(qgCurrency) + sum7);
                headMap.put("sum8", currencyFuhaoMap.get(yfCurrency) + sum8);
                headMap.put("sum9", currencyFuhaoMap.get(qgCurrency) + sum9);
                headMap.put("sum10", currencyFuhaoMap.get(yfCurrency) + sum10);
                headMap.put("sum11", currencyFuhaoMap.get(qgCurrency) + sum11);
                headMap.put("sum12", currencyFuhaoMap.get(yfCurrency) + sum12);
                headMap.put("sum13", currencyFuhaoMap.get(qgCurrency) + sum13);

                //按模板中的逻辑计算出来的
                headMap.put("DesAgentFee",
                        currencyFuhaoMap.get(yfCurrency) +
                                sum6.multiply(new BigDecimal(0.2)).
                                        setScale(2, RoundingMode.HALF_UP));
                headMap.put("BillinTTl",
                        currencyFuhaoMap.get(yfCurrency)
                                + sum6.subtract(sum4).subtract(
                                sum6.multiply(new BigDecimal(0.2)).setScale(2, RoundingMode.HALF_UP)));

                int sumCount = 0;
                for (BoxMergePkgDO boxMergePkgDO : boxMergePkgDOS) {
                    Long count = boxPkgOrderService.selectCount(BoxPkgOrderDO::getPkgId, boxMergePkgDO.getId());
                    if (count != null && count > 0L) {
                        sumCount = sumCount + 1;
                    }
                }
                headMap.put("sum14", sum14 + sumCount);
                headMap.put("sum15", sum15);
                headMap.put("sum16", sum16);
                StringBuffer stringBuffer = new StringBuffer();
                if (otherMainCurrency.intValue() == otherAssisCurrency.intValue()) {
                    stringBuffer.append(currencyMap.get(otherMainCurrency)).append("：").append(sum4);
                } else {
                    stringBuffer.append(currencyMap.get(otherMainCurrency)).append("：").append(sum4).append("     ").append(currencyMap.get(otherAssisCurrency)).append("：").append(sum5);
                }
                headMap.put("otherFee", stringBuffer.toString());
                headMap.put("otherFeeMain", currencyMap.get(otherMainCurrency)+"："+sum4);
                headMap.put("otherFeeAssis", currencyMap.get(otherAssisCurrency)+"："+sum5);
                File fileDir = new File(dir);
                if (!fileDir.exists()) {
                    // 不存在则创建一个目录
                    fileDir.mkdirs();
                }
                ExcelWriter excelWriter =
                        EasyExcel.write(dir + fileName).
                                withTemplate(inputStream).build();

                WriteSheet sheet = EasyExcel.writerSheet(0).build();
                FillConfig fillConfig = FillConfig.builder().forceNewRow(true).build();
                excelWriter.fill(headMap, sheet);//填充非循环数据--表头

                excelWriter.fill(list, fillConfig, sheet);//填充循环数据--表数据
                excelWriter.finish();
                inputStream.close();

                // 获取到临时文件
                File file = new File(dir + fileName);
                //TODO 利用POI 合并单元格
                Workbook workbook = new XSSFWorkbook(file);
                Sheet poiSheet = workbook.getSheetAt(0);
                if (otherMainCurrency.intValue() == otherAssisCurrency.intValue()) {
                    poiSheet.addMergedRegion(new CellRangeAddress(11, 11, 8, 9));
                    for (int i = 1; i <= orderMap.size(); i++) {
                        poiSheet.addMergedRegion(new CellRangeAddress(11 + i, 11 + i, 8, 9));
                    }
                    poiSheet.addMergedRegion(new CellRangeAddress(12 + orderMap.size(), 12 + orderMap.size(), 8, 9));
                }

                if (CollectionUtil.isNotEmpty(boxMergePkgDOS)) {
                    int start = 12;
                    for (int i = 0; i < boxMergePkgDOS.size(); i++) {
                        int count = boxPkgOrderService.selectList(new LambdaQueryWrapperX<BoxPkgOrderDO>().eq(BoxPkgOrderDO::getPkgId, boxMergePkgDOS.get(i).getId()).groupBy(BoxPkgOrderDO::getOrderId)).size();
                        if (count > 1L) {
                            poiSheet.addMergedRegion(new CellRangeAddress(start, start + count - 1, 18, 18));
                            poiSheet.addMergedRegion(new CellRangeAddress(start, start + count - 1, 27, 27));
                        }
                        start = start + count;
                    }
                }
                //业务数据从12行开始,22列为是否控货
                Font font = workbook.createFont();
                font.setColor(Font.COLOR_RED); //红色
                CellStyle cellStyle=workbook.createCellStyle();
                cellStyle.setFont(font);
                cellStyle.setAlignment(HorizontalAlignment.CENTER);
                cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
                cellStyle.setBorderLeft(BorderStyle.THIN);
                cellStyle.setBorderBottom(BorderStyle.THIN);
                cellStyle.setBorderRight(BorderStyle.THIN);
                cellStyle.setBorderTop(BorderStyle.THIN);
                for (int i = 0; i < list.size(); i++) {
                    if (poiSheet.getRow(i+12).getCell(22).getStringCellValue().equals("YES")) {
                        for (int i1 = 0; i1 < 29; i1++) {
                            poiSheet.getRow(i+12).getCell(i1).setCellStyle(cellStyle);
                        }
                    }
                }

                FileOutputStream fileOutputStream =
                        new FileOutputStream(new File(dir + fileNameMerge));
                workbook.write(fileOutputStream);
                fileOutputStream.close();

                File mergeFile = new File(dir + fileNameMerge);
                // 创建FileInputStream对象
                FileInputStream fileInputStream = new FileInputStream(mergeFile);
                // 读取文件内容
                byte[] fileBytes = new byte[(int) mergeFile.length()];
                fileInputStream.read(fileBytes);
                // 关闭文件流
                fileInputStream.close();
                // 将文件上传到资源服务器
                FileDO fileDO = fileService.createFile(dir, fileNameMerge, fileBytes);
                event.setPath(fileDO.getPath());
                event.setFileName(fileDO.getPath());
                event.setUrl(fileDO.getUrl());
                event.setFileId(fileDO.getId());

            } catch (Exception e) {
                e.printStackTrace();
                event.setResult(e.getMessage());
            }
        } else {
            event.setResult("param fail");
        }
    }


    private BigDecimal changeAmountToDestCurrency(Long fromCurrencyId,
                                                  Long toCurrencyId,
                                                  BigDecimal fromAmount) {
        BigDecimal destAmount = BigDecimal.ZERO;
        ExchangeRateRespDTO currencyRate = currencyApi.getCurrencyRate(fromCurrencyId, toCurrencyId);
        destAmount = fromAmount.multiply(currencyRate.getCurrencyRate());
        return destAmount;
    }

    private String getUserName(String userId) {
        if (StringUtils.isEmpty(userId)) {
            return null;
        }
        return adminUserApi.getUser(Long.valueOf(userId)).getNickname();
    }
}

